#<profile4xi profile = "02" /> 
# -------------------------------------------------------------------------------------------------
#<program4xi program = "synchronizedstring4xi.py">
#<abstract4xi abstract = "provides class SynchronizedString4Xi"/>
#<refer4xi reference = "sharedctypes.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'SynchronizedString4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import weakref         
except:
    pass
try:                 # rare ones
    import ctypes          
except:
    pass 
try:                 # imported-as ones 
    pass          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    pass
except:
    pass
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from make_property4xi import make_property4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from synchronizedarray4xi import SynchronizedArray4Xi
from print4xi import print4xi
from time4xi import time4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
prop_cache = {}
class_cache = weakref.WeakKeyDictionary()
#"
#"/>

# -------------------------------------------------------------------------------------------------
# Types                              
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/> 
# -------------------------------------------------------------------------------------------------
#
#<type4xi type = "
typecode_to_type = {
    'c': ctypes.c_char,  'u': ctypes.c_wchar,
    'b': ctypes.c_byte,  'B': ctypes.c_ubyte,
    'h': ctypes.c_short, 'H': ctypes.c_ushort,
    'i': ctypes.c_int,   'I': ctypes.c_uint,
    'l': ctypes.c_long,  'L': ctypes.c_ulong,
    'f': ctypes.c_float, 'd': ctypes.c_double}
#"

#usage = "
#    typecodes to ctypes mapping
#"
#>



# =================================================================================================
#<class4xi class = "SynchronizedString4Xi" base = "SynchronizedArray4Xi" child = "None" 
#   synopsis = "synchronized string services"
#   usage = "
#     sharedctypes.py
#         synchronized(obj, lock=None): 
#             if obj._type_ is ctypes.c_char:
#                 return SynchronizedString(obj, lock)
#
#     sharectype.py
#         SynchronizedString(SynchronizedArray): <--- no methods!
#             value = make_property('value') <-- class attributes
#             raw = make_property('raw')
#
#     synchronized_function4xi.py:
#         return SynchronizedString4Xi(obj, lock) 
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class SynchronizedString4Xi (SynchronizedArray4Xi):
    '''
    synchronized string services
    '''
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    value = make_property4xi('value') 
    raw = make_property4xi('raw')
    #"
    #/>
        
    #<note4xi note = "__init__ and normal methods are missing"/>    
    
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-SynchronizedString4Xi" usage = "senses SynchronizedString4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses class objects
        '''
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
    
        
        class2print = 'SynchronizedString4Xi'
        method2print = 'sense'
        
        message2print = "{started" + "; sense-of-SynchronizedString4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; SynchronizedString4Xi-sensing-by-sense-of-SynchronizedString4Xi: " + str(SynchronizedString4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-SynchronizedString4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['value-of-SynchronizedString4Xi'] = SynchronizedString4Xi.value    
            kb2xi['raw-of-SynchronizedString4Xi'] = SynchronizedString4Xi.raw           
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-SynchronizedString4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
      
        message2print = "{sensed" + "; SynchronizedString4Xi-sensed-by-sense-of-SynchronizedString4Xi: " + str(SynchronizedString4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-SynchronizedString4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        

    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-SynchronizedString4Xi" usage = "reasons SynchronizedString4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons SynchronizedString4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
    
        
        class2print = 'SynchronizedString4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-SynchronizedString4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; SynchronizedString4Xi-reasoning-by-reason-of-SynchronizedString4Xi: " + str(SynchronizedString4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-SynchronizedString4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; SynchronizedString4Xi-reasoned-by-reason-of-SynchronizedString4Xi: " + str(SynchronizedString4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-SynchronizedString4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                            
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-SynchronizedString4Xi" usage = "aces SynchronizedString4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces SynchronizedString4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
   
        
        
        class2print = 'SynchronizedString4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-SynchronizedString4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
             
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; SynchronizedString4Xi-acing-by-ace-of-SynchronizedString4Xi: " + str(SynchronizedString4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                   
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-SynchronizedString4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; SynchronizedString4Xi-aced-by-ace-of-SynchronizedString4Xi: " + str(SynchronizedString4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-SynchronizedString4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                             
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of synchronizedstring4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
